import os
from colorama import init, Fore, Style
from IMDLBenCo.utils.paths import BencoPath
from .copy import copy_files, copy_file 

def cli_guide(config):
    print("This is guide cli.")
    # TODO